﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Threading;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

namespace Sample1
{
    public partial class Form1 : Form
    {
        public UInt32 DeviceIndex;
        private bool terminated;
        private long t1;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Called before Form is displayed Initialize resources:
            DeviceIndex = AIOUSB.diOnly;

            UInt32 ERROR_SUCCESS = 0;
            // Device data:
            UInt32 Status = ERROR_SUCCESS;
            UInt32 PID = 0;
            Int32 NameSize = 256;
            UInt64 SerNum = 0;
            int deviceCount = 0;

            // Get The Device Information test for devices found:
            for (DeviceIndex = 0; DeviceIndex < 32; DeviceIndex++)
            {
                Status = AIOUSB.QueryDeviceInformation(DeviceIndex, out PID, out NameSize, out string strName, out UInt32 DIOBytes, out UInt32 Counters);
                if (Status != ERROR_SUCCESS) continue;

                Status = AIOUSB.GetDeviceSerialNumber(DeviceIndex, out SerNum);
                if (Status != ERROR_SUCCESS)
                    SerNum = 0;

                DeviceListBox.Items.Add("Device at DeviceIndex[" + DeviceIndex.ToString() + "] is a '" + strName + "' ( SN= " + SerNum.ToString("X") + " )");

                deviceCount++;
            }
            DeviceListBox.Items.Add(string.Format("{0} device{1} detected.", deviceCount, (deviceCount != 1) ? "s" : ""));
            if (deviceCount == 0)
                DeviceListBox.Items.Add("Please check Device Manager to confirm your devices are properly installed.");
            DeviceListBox.Items.Add("Done.");
        }

        private void butConfigure_Click(object sender, EventArgs e)
        {
            UInt32 ConfigSize = 2;
            byte[] config = { 1, 1 };
            UInt32 Status;
            DeviceIndex = AIOUSB.diOnly;
            Status = AIOUSB.ADC_SetConfig(DeviceIndex, config, ref ConfigSize);
            // handle status
            textBoxStatusLog.Text += "AIOUSB.ADC_SetConfig status: " + Status.ToString();
            if (0 != Status) return;
            but1Scan.Enabled = true;
        }

        private void ADCStart()
        {
            byte[] b= { 0 }; // unused
            AIOUSB.GenericVendorWrite(DeviceIndex, 0xBC, 0x03, 0, 0, ref b);
        }
        private void ADCStop()
        {
            byte[] b = { 0 }; // unused
            AIOUSB.GenericVendorWrite(DeviceIndex, 0xBC, 0x00, 0, 0, ref b);
        }


        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceCounter(out long lpPerformanceCount);

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceFrequency(out long lpFrequency);


        private void but1Scan_Click(object sender, EventArgs e)
        {
            QueryPerformanceCounter(out long t1);
            ADCStart();

            int BytesLeft = 512;
            byte[] buf = new byte[1024];
            do
            {
                var Status = AIOUSB.AWU_GenericBulkIn(DeviceIndex, 0x86, Marshal.UnsafeAddrOfPinnedArrayElement(buf, 0), 512, out Int32 bytesTaken);
                if (Status != 0)
                    terminated = true;
                BytesLeft -= bytesTaken;

            } while (!terminated && (BytesLeft > 0));

            ADCStop();
            QueryPerformanceCounter(out long t2);
            long dT = t2 - t1;

            UpdateDisplay(buf, t1, dT);
        }

        private void UpdateDisplay(byte[] scanBuf, long t1, long dT)
        {
            int byteIndex = 0;
            UInt16 count0, count1;
            UInt64 accCh0 = 0, accCh1 = 0;
            double v0, v1;
            textBoxStatusLog.AppendText("1 Scan in " + dT.ToString() + " µs\r\n");
            listBoxScanData.Items.Clear();
            while (byteIndex+8 <= 512) 
            {
                count0 = (UInt16)(scanBuf[byteIndex + 0] + (scanBuf[byteIndex + 1] << 8)); byteIndex += 2;
                count1 = (UInt16)(scanBuf[byteIndex + 0] + (scanBuf[byteIndex + 1] << 8)); byteIndex += 2;
                UInt32 statusAndTick = (UInt32)(scanBuf[byteIndex + 0] + (scanBuf[byteIndex + 1] << 8) + (scanBuf[byteIndex + 2] << 16) + (scanBuf[byteIndex + 3] << 24)); byteIndex += 4;

                listBoxScanData.Items.Add( $"ADC Ch 0: {count0:x4}, ADC Ch 1: {count1:x4}, statTick:{statusAndTick:X8}");

                accCh0 += count0;
                accCh1 += count1;
            };
            v0 = (accCh0 / 64.0 * 20.0 ) / 65536.0 - 10.0;
            v1 = (accCh1 / 64.0 * 20.0 ) / 65536.0 - 10.0;

            textBoxStatusLog.Text += $"ADC ch0: {v0:f4}\r\nADC ch1: {v1:f4}\r\n";
        }

        private void butScan_Click(object sender, EventArgs e)
        {
            Thread.CurrentThread.Priority = ThreadPriority.Highest;
            do
            {
                QueryPerformanceCounter(out long t1);
                ADCStart();

                int BytesLeft = 512;
                byte[] buf = new byte[1024];
                do
                {
                    var Status = AIOUSB.AWU_GenericBulkIn(DeviceIndex, 0x86, Marshal.UnsafeAddrOfPinnedArrayElement(buf, 0), 512, out Int32 bytesTaken);
                    if (Status != 0)
                        terminated = true;
                    BytesLeft -= bytesTaken;

                } while (!terminated && (BytesLeft > 0));

                ADCStop();
                QueryPerformanceCounter(out long t2);
                long dT = t2 - t1;

                Application.DoEvents();

            } while (!terminated);
            terminated = false;
            Thread.CurrentThread.Priority = ThreadPriority.Normal;
        }
    }
}
